# AUTOGENERATED! DO NOT EDIT! File to edit: ../02_datasets.ipynb.

# %% auto 0
__all__ = ['ReutersParser', 'ReutersCorpus', 'build_reuters_dataframe']

# %% ../02_datasets.ipynb 3
from io import StringIO
from collections import Counter
import xml.etree.ElementTree as _et

# %% ../02_datasets.ipynb 4
class ReutersParser:
    _broken = (
        b"&#1;",
        b"&#2;",
        b"&#3;",
        b"\xfc",
        b"&#5;",
        b"&#22;",
        b"&#27;",
        b"&#30;",
        b"&#31;",
        b"&#127;",
    )

    def cleanup_sgml(self, chunk):
        for item in self._broken:
            chunk = chunk.replace(item, b"")
        chunk = chunk.replace(b'<!DOCTYPE lewis SYSTEM "lewis.dtd">', b"<document>")
        return b"%s</document>" % chunk

    def get_topics(self, topics):
        return [topic.text for topic in topics]

    def get_text(self, text):
        tagmap = dict.fromkeys(("title", "dateline", "body"))
        for item in text:
            tag = item.tag.lower()
            if tag in tagmap:
                tagmap[tag] = item.text
        return tagmap

    def parse_doc(self, elem):
        doc = {}
        doc["attrs"] = dict(elem.items())
        for item in elem:
            if item.tag == "TOPICS":
                doc["topics"] = self.get_topics(item)
            elif item.tag == "DATE":
                doc["date"] = item.text
            elif item.tag == "TEXT":
                doc.update(self.get_text(item))
        return doc

    def parse_sgml(self, filename):
        stream = StringIO(
            self.cleanup_sgml(open(filename, "rb").read()).decode("utf-8")
        )
        for _, elem in _et.iterparse(stream):
            if elem.tag == "REUTERS":
                yield self.parse_doc(elem)

class ReutersCorpus:
    def __init__(self, raw_docs, multiclass=False, filter_empty_cats=True):
        self.topics = {}
        self.cat_to_topic = {}
        self.target_names = []

        self.docs = list(self.get_docs(raw_docs))
        if multiclass:
            self.docs = self.filter_multi_label(self.docs)

        # before filtering empty cats it's 9603 train and 3299 test documents
        # after filtering there are 7770 train and 3019 test documents left
        if filter_empty_cats:
            self.docs = self.filter_empty_cats(self.docs)

        # labels have to be without gaps
        self._renumber_topics()

    def _renumber_topics(self):
        self.renumbered_cat_to_topic = {}
        self.renumbered_topics = {}
        self.renumbered_lookup = {}
        num = 0
        for doc in self.docs:
            new_cats = []
            for cat in doc["cats"]:
                if cat not in self.renumbered_lookup:
                    topic = self.cat_to_topic[cat]
                    self.renumbered_cat_to_topic[num] = topic
                    self.renumbered_topics[topic] = num
                    self.renumbered_lookup[cat] = num
                    num += 1
                new_cat = self.renumbered_lookup[cat]
                new_cats.append(new_cat)
            doc["cats"] = new_cats

    def _add_text(self, doc):
        # doc["text"] = " ".join([doc.get(tag) or "" for tag in
        #    ("title", "dateline", "body")])
        doc["text"] = " ".join([doc.get(tag) or "" for tag in ("dateline", "body")])
        title = " ".join([doc.get("title") or "" for i in range(1)])
        doc["text"] = "%s %s" % (title, doc["text"])

    def _add_modapte(self, doc):
        attrs = doc["attrs"]
        doc["modapte"] = "unused"
        if attrs["LEWISSPLIT"] == "TRAIN" and attrs["TOPICS"] == "YES":
            doc["modapte"] = "train"
        elif attrs["LEWISSPLIT"] == "TEST" and attrs["TOPICS"] == "YES":
            doc["modapte"] = "test"

    def _add_topics(self, doc):
        doc["cats"] = []
        for topic in doc["topics"]:
            if topic not in self.topics:
                self.target_names.append(topic)
                topic_id = len(self.target_names)
                self.topics[topic] = topic_id
                self.cat_to_topic[topic_id] = topic
            topic_id = self.topics[topic]
            doc["cats"].append(topic_id)

    def get_docs(self, documents):
        modifiers = [self._add_text, self._add_modapte, self._add_topics]
        for doc in documents:
            for modifier in modifiers:
                modifier(doc)
            if doc["modapte"] != "unused":
                yield doc

    def filter_empty_cats(self, docs):
        # modapte yields 90 categories with 1 train and test doc at least
        train, test = set(), set()
        for doc in docs:
            if doc["modapte"] == "train":
                for cat in doc["cats"]:
                    train.add(cat)
            elif doc["modapte"] == "test":
                for cat in doc["cats"]:
                    test.add(cat)
        valid_cats = train.intersection(test)
        self.valid_cats = valid_cats
        new_docs = []
        for doc in docs:
            doc["cats"] = [c for c in doc["cats"] if c in valid_cats]
            if len(doc["cats"]) > 0:
                new_docs.append(doc)
        return new_docs

    def filter_multi_label(self, docs):
        filtered_docs = []
        for doc in docs:
            if len(doc["cats"]) == 1:
                filtered_docs.append(doc)
        return filtered_docs

    @property
    def number_of_samples(self):
        return len(self.docs)

    @property
    def number_of_classes(self):
        return len([name for name, count in self.topics.items() if count > 1])

    @property
    def texts(self):
        return [d["text"] for d in self.docs]

    @property
    def topic_counts(self):
        counts = Counter()
        for doc in self.docs:
            for topic in doc["cats"]:
                counts[topic] += 1
        return counts

    def top_n(self, n=10):
        topic_lookup = {v: k for k, v in self.renumbered_topics.items()}
        top_topics = sorted(
            [(v, k) for k, v in self.topic_counts.items()], reverse=True
        )[:n]
        top_n_topics = [
            (topic_lookup[topic_id], topic_id) for (count, topic_id) in top_topics[:n]
        ]
        top_n_ids = [topic_id for (name, topic_id) in top_n_topics]
        top_n_names = [name for name, topic_id in top_n_topics]
        return top_n_ids, top_n_names

    def get_single_label(self, docs, top_n):
        labels = []
        for doc in docs:
            # default label is the first one
            label = doc["cats"][0]
            for cat in doc["cats"]:
                if cat in top_n:
                    label = cat
            labels.append(label)
        return labels

    def get_labels(self, docs):
        labels = []
        for doc in docs:
            labels.append(doc["cats"])
        return labels

    def split_modapte(self):
        train, test = [], []
        for doc in self.docs:
            if doc["modapte"] == "train":
                train.append(doc)
            elif doc["modapte"] == "test":
                test.append(doc)
        return train, test

    def build_dataframe(self, pd=None, n=10):
        if pd is None:
            # the df parameter is only there to avoid a hard dependency to pandas
            return None
        top_ten_ids, top_ten_names = self.top_n(n=n)
        train_docs, test_docs = self.split_modapte()
        docs = train_docs + test_docs
        train_labels = self.get_labels(train_docs)
        test_labels = self.get_labels(test_docs)
        labels = train_labels + test_labels

#        labels = train_labels + test_labels
#        label_lookup = {}
#        num = 0
#        for label in sorted(labels):
#            if label not in label_lookup:
#                label_lookup[label] = num
#                num += 1
#
#        topic_lookup = {v: k for k, v in self.topics.items()}
#        orig_labels = [topic_lookup[l] for l in labels]
#
#        labels = [label_lookup[l] for l in labels]
#        train_labels = [label_lookup[l] for l in train_labels]
#        test_labels = [label_lookup[l] for l in test_labels]
#        top_ten_ids = [label_lookup[tid] for tid in top_ten_ids]

        orig_labels = []
        for cats in labels:
            topics = [self.renumbered_cat_to_topic[c] for c in cats]
            orig_labels.append(topics)

        # build dataframe
        df = pd.DataFrame()
        df["modapte"] = [d["modapte"] for d in docs]
        df["category"] = orig_labels
        df["label"] = train_labels + test_labels
        df["date"] = [d["date"] for d in docs]
        df["title"] = [d["title"] for d in docs]
        df["dateline"] = [d["dateline"] for d in docs]
        df["body"] = [d["body"] for d in docs]
        df["newid"] = [d["attrs"]["NEWID"] for d in docs]
        df["date"] = pd.to_datetime(
            df.date.str.split(".").apply(lambda x: x[0].lstrip()),
            format="%d-%b-%Y %H:%M:%S",
        )
        df["wd_name"] = df.date.dt.day_name()
        return df, top_ten_ids, train_labels, test_labels


def build_reuters_dataframe(docs, topics, train_labels, test_labels, top_ten_ids, pd=None):
    if pd is None:
        # the df parameter is only there to avoid a hard dependency to pandas
        return None    
    # remove gaps
    labels = train_labels + test_labels
    label_lookup = {}
    num = 0
    for label in sorted(labels):
        if label not in label_lookup:
            label_lookup[label] = num
            num += 1

    topic_lookup = {v: k for k, v in topics.items()}
    orig_labels = [topic_lookup[l] for l in labels]

    labels = [label_lookup[l] for l in labels]
    train_labels = [label_lookup[l] for l in train_labels]
    test_labels = [label_lookup[l] for l in test_labels]
    top_ten_ids = [label_lookup[tid] for tid in top_ten_ids]

    # build dataframe
    df = pd.DataFrame()
    df["modapte"] = [d["modapte"] for d in docs]
    df["category"] = orig_labels
    df["label"] = train_labels + test_labels
    df["date"] = [d["date"] for d in docs]
    df["title"] = [d["title"] for d in docs]
    df["dateline"] = [d["dateline"] for d in docs]
    df["body"] = [d["body"] for d in docs]
    df["newid"] = [d["attrs"]["NEWID"] for d in docs]
    df["date"] = pd.to_datetime(
        df.date.str.split(".").apply(lambda x: x[0].lstrip()),
        format="%d-%b-%Y %H:%M:%S",
    )
    df["wd_name"] = df.date.dt.day_name()
    return df, top_ten_ids, train_labels, test_labels



# AUTOGENERATED! DO NOT EDIT! File to edit: ../00_setup.ipynb.

# %% auto 0
__all__ = ['running_in_colab', 'setup_colab', 'set_retina_mode', 'setup']

# %% ../00_setup.ipynb 2
import IPython

# %% ../00_setup.ipynb 3
def running_in_colab():
    """
    Check whether we are running in a Google Colab notebook.
    """
    try:
        if 'google.colab' in str(get_ipython()):
          return True
    except Exception:
        pass
    return False


def setup_colab():
    "Sets up Colab"
    assert running_in_colab(), "You do not appear to be running in Colab"


def set_retina_mode():
    """
    Does the same thing as this `%config` magic:

    %config InlineBackend.figure_format = 'retina'
    """
    shell = IPython.get_ipython()
    if shell is not None:
        shell.config.InlineBackend.figure_format = "retina"


def setup():
    "Sets up ds_tut. First run `%pip install -Uqq ds_tut` in a cell"
    if running_in_colab():
        setup_colab()
    set_retina_mode()



# AUTOGENERATED! DO NOT EDIT! File to edit: ../01_io.ipynb.

# %% auto 0
__all__ = ['download_from_url']

# %% ../01_io.ipynb 3
import httpx
from pathlib import Path

import rich.progress

# %% ../01_io.ipynb 4
def download_from_url(
    url: str, # url to download
    dst: Path, # the path of the destination file
    chunk_size=2048 # optional chunk_size for response.iter_bytes
) -> int:  # the file size
    if dst.exists():
        file_size = dst.stat().st_size
        if file_size > 0:
            # return early if file was already downloaded
            return file_size
        
    with dst.open("ab") as download_file:
        with httpx.stream("GET", url) as response:
            total = int(response.headers["Content-Length"])
    
            with rich.progress.Progress(
                "[progress.percentage]{task.percentage:>3.0f}%",
                rich.progress.BarColumn(bar_width=None),
                rich.progress.DownloadColumn(),
                rich.progress.TransferSpeedColumn(),
                refresh_per_second=5,
            ) as progress:
                download_task = progress.add_task("Download", total=total)
                for chunk in response.iter_bytes(chunk_size=chunk_size):
                    download_file.write(chunk)
                    progress.update(download_task, completed=response.num_bytes_downloaded, refresh=True)
    return response.num_bytes_downloaded



# AUTOGENERATED! DO NOT EDIT! File to edit: ../03_transformers.ipynb.

# %% auto 0
__all__ = ['EmptyFitMixin']

# %% ../03_transformers.ipynb 4
class EmptyFitMixin:
    def fit(self, x, y=None):
        return self



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/ds_tut',
                'doc_host': 'https://ephes.github.io',
                'git_url': 'https://github.com/ephes/ds_tut',
                'lib_path': 'ds_tut'},
  'syms': { 'ds_tut.core': {'ds_tut.core.foo': ('core.html#foo', 'ds_tut/core.py')},
            'ds_tut.datasets': { 'ds_tut.datasets.ReutersCorpus': ('datasets.html#reuterscorpus', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.__init__': ('datasets.html#reuterscorpus.__init__', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus._add_modapte': ( 'datasets.html#reuterscorpus._add_modapte',
                                                                                 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus._add_text': ('datasets.html#reuterscorpus._add_text', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus._add_topics': ( 'datasets.html#reuterscorpus._add_topics',
                                                                                'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus._renumber_topics': ( 'datasets.html#reuterscorpus._renumber_topics',
                                                                                     'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.build_dataframe': ( 'datasets.html#reuterscorpus.build_dataframe',
                                                                                    'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.filter_empty_cats': ( 'datasets.html#reuterscorpus.filter_empty_cats',
                                                                                      'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.filter_multi_label': ( 'datasets.html#reuterscorpus.filter_multi_label',
                                                                                       'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.get_docs': ('datasets.html#reuterscorpus.get_docs', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.get_labels': ( 'datasets.html#reuterscorpus.get_labels',
                                                                               'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.get_single_label': ( 'datasets.html#reuterscorpus.get_single_label',
                                                                                     'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.number_of_classes': ( 'datasets.html#reuterscorpus.number_of_classes',
                                                                                      'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.number_of_samples': ( 'datasets.html#reuterscorpus.number_of_samples',
                                                                                      'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.split_modapte': ( 'datasets.html#reuterscorpus.split_modapte',
                                                                                  'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.texts': ('datasets.html#reuterscorpus.texts', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.top_n': ('datasets.html#reuterscorpus.top_n', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersCorpus.topic_counts': ( 'datasets.html#reuterscorpus.topic_counts',
                                                                                 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersParser': ('datasets.html#reutersparser', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersParser.cleanup_sgml': ( 'datasets.html#reutersparser.cleanup_sgml',
                                                                                 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersParser.get_text': ('datasets.html#reutersparser.get_text', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersParser.get_topics': ( 'datasets.html#reutersparser.get_topics',
                                                                               'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersParser.parse_doc': ('datasets.html#reutersparser.parse_doc', 'ds_tut/datasets.py'),
                                 'ds_tut.datasets.ReutersParser.parse_sgml': ( 'datasets.html#reutersparser.parse_sgml',
                                                                               'ds_tut/datasets.py'),
                                 'ds_tut.datasets.build_reuters_dataframe': ( 'datasets.html#build_reuters_dataframe',
                                                                              'ds_tut/datasets.py')},
            'ds_tut.io': {'ds_tut.io.download_from_url': ('io.html#download_from_url', 'ds_tut/io.py')},
            'ds_tut.setup': { 'ds_tut.setup.running_in_colab': ('setup.html#running_in_colab', 'ds_tut/setup.py'),
                              'ds_tut.setup.set_retina_mode': ('setup.html#set_retina_mode', 'ds_tut/setup.py'),
                              'ds_tut.setup.setup': ('setup.html#setup', 'ds_tut/setup.py'),
                              'ds_tut.setup.setup_colab': ('setup.html#setup_colab', 'ds_tut/setup.py')},
            'ds_tut.transformers': { 'ds_tut.transformers.EmptyFitMixin': ('transformers.html#emptyfitmixin', 'ds_tut/transformers.py'),
                                     'ds_tut.transformers.EmptyFitMixin.fit': ( 'transformers.html#emptyfitmixin.fit',
                                                                                'ds_tut/transformers.py')}}}
